package chess4j.search;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import chess4j.Constants;
import chess4j.board.Board;
import chess4j.io.PrintGameResult;
import chess4j.io.PrintLine;
import chess4j.moves.Move;
import chess4j.utils.GameStatus;
import chess4j.utils.GameStatusChecker;
import chess4j.utils.MoveUtils;

/**
 * SearchIterator
 * @author james
 *
 */
public class SearchIterator implements Runnable {

	private static final Log logger = LogFactory.getLog(SearchIterator.class);
	private static final SearchIterator INSTANCE = new SearchIterator();
	private static int maxDepth = 0;
	private static int timeRemaining,increment;
	
	private SearchIterator() {
	}
	
	public static SearchIterator getInstance() {
		return INSTANCE;
	}
	
	private void iterate() {
		Board board = Board.INSTANCE.deepCopy();
		List<Move> pv = new ArrayList<Move>();
		
		SearchStats stats = new SearchStats();
		stats.setStartTime(System.currentTimeMillis());
		int allotedTime = getTimeAllotment();
		logger.debug("# alloted time: " + allotedTime);
		stats.setStopTime(stats.getStartTime()+allotedTime);
		
		boolean stopSearching = false;
		int depth = 0;
		
		do {
			++depth;
			int score=Search.search(pv,-Constants.INFINITY, Constants.INFINITY, board, depth,stats);
			assert(pv.size()>0);
			if (Search.isAbortSearch()) {
				break;
			}
			PrintLine.printLine(pv,depth,score,stats.getStartTime(),stats.getNodes());
			//System.out.print("first line: ");
			//PrintLine.printLine(stats.getFirstLine());
			stats.setLastPV(pv);
			
			if (maxDepth > 0 && depth >= maxDepth) {
				logger.debug("# stopping iterative search on depth");
				stopSearching = true;
			}
			
			// if we've used more than half our allotted time, don't start a new iteration.
			long elapsedTime = System.currentTimeMillis() - stats.getStartTime();
			if (elapsedTime > (allotedTime / 2)) {
				logger.debug("# stopping iterative search because half time expired.");
				stopSearching = true;
			}
		} while (!stopSearching);
		
		assert(pv.size()>0);
		assert(MoveUtils.isLineValid(pv, Board.INSTANCE));
		
		Move m = pv.get(0);
		Board.INSTANCE.applyMove(m);
		logger.info("move " + m);
	
		// if end of game print result
		GameStatus gs = GameStatusChecker.getGameStatus(Board.INSTANCE);
		if (gs != GameStatus.INPROGRESS) {
			PrintGameResult.printResult(gs);
		}
	}
	
	
	public static int getMaxDepth() {
		return maxDepth;
	}

	public static void setMaxDepth(int maxDepth) {
		SearchIterator.maxDepth = maxDepth;
	}
	

	public static int getTimeRemaining() {
		return timeRemaining;
	}

	public static void setTimeRemaining(int timeRemaining) {
		SearchIterator.timeRemaining = timeRemaining;
	}

	public static int getIncrement() {
		return increment;
	}

	public static void setIncrement(int increment) {
		SearchIterator.increment = increment;
	}

	public static int getTimeAllotment() {
		if (timeRemaining < 0) {
			timeRemaining = 0;
		}
		return (timeRemaining / 25) + increment;
	}
	
	public void run() {
		iterate();
	}
}
