package chess4j.board;

import org.junit.Test;

import junit.framework.Assert;

public class SquareTest {

	@Test
	public void getValue() {
		Assert.assertEquals(0, Square.valueOf(File.FILE_A, Rank.RANK_8).value());
		Assert.assertEquals(7, Square.valueOf(File.FILE_H, Rank.RANK_8).value());
		Assert.assertEquals(56, Square.valueOf(File.FILE_A, Rank.RANK_1).value());
		Assert.assertEquals(63, Square.valueOf(File.FILE_H, Rank.RANK_1).value());
		Assert.assertEquals(42, Square.valueOf(File.FILE_C, Rank.RANK_3).value());
		Assert.assertEquals(36, Square.valueOf(File.FILE_E, Rank.RANK_4).value());
	}
	
	@Test
	public void testToString() {
		Assert.assertEquals("a8", Square.valueOf(File.FILE_A, Rank.RANK_8).toString());
		Assert.assertEquals("h8", Square.valueOf(File.FILE_H, Rank.RANK_8).toString());
		Assert.assertEquals("a1", Square.valueOf(File.FILE_A, Rank.RANK_1).toString());
		Assert.assertEquals("h1", Square.valueOf(File.FILE_H, Rank.RANK_1).toString());
		Assert.assertEquals("c3", Square.valueOf(File.FILE_C, Rank.RANK_3).toString());
		Assert.assertEquals("e4", Square.valueOf(File.FILE_E, Rank.RANK_4).toString());
	}
	
	@Test
	public void testFileSquares() {
		Assert.assertTrue(Square.fileSquares(File.FILE_B).contains(Square.valueOf(File.FILE_B, Rank.RANK_2)));
		Assert.assertFalse(Square.fileSquares(File.FILE_B).contains(Square.valueOf(File.FILE_C, Rank.RANK_2)));
	}
	
	@Test
	public void testRankSquares() {
		Assert.assertTrue(Square.rankSquares(Rank.RANK_2).contains(Square.valueOf(File.FILE_B, Rank.RANK_2)));
		Assert.assertFalse(Square.rankSquares(Rank.RANK_2).contains(Square.valueOf(File.FILE_B, Rank.RANK_3)));
	}

	@Test
	public void testFlipVertical() {
		Assert.assertEquals(Square.valueOf(File.FILE_A, Rank.RANK_8), Square.valueOf(File.FILE_A, Rank.RANK_1).flipVertical());
		Assert.assertEquals(Square.valueOf(File.FILE_E, Rank.RANK_2), Square.valueOf(File.FILE_E, Rank.RANK_7).flipVertical());
		Assert.assertEquals(Square.valueOf(File.FILE_C, Rank.RANK_6), Square.valueOf(File.FILE_C, Rank.RANK_3).flipVertical());
		Assert.assertEquals(Square.valueOf(File.FILE_B, Rank.RANK_5), Square.valueOf(File.FILE_B, Rank.RANK_4).flipVertical());
		Assert.assertEquals(Square.valueOf(File.FILE_H, Rank.RANK_7), Square.valueOf(File.FILE_H, Rank.RANK_2).flipVertical());
	}
	
	@Test
	public void testFlipHorizontal() {
		Assert.assertEquals(Square.valueOf(File.FILE_A, Rank.RANK_8), Square.valueOf(File.FILE_H, Rank.RANK_8).flipHorizontal());
		Assert.assertEquals(Square.valueOf(File.FILE_E, Rank.RANK_2), Square.valueOf(File.FILE_D, Rank.RANK_2).flipHorizontal());
		Assert.assertEquals(Square.valueOf(File.FILE_C, Rank.RANK_6), Square.valueOf(File.FILE_F, Rank.RANK_6).flipHorizontal());
		Assert.assertEquals(Square.valueOf(File.FILE_B, Rank.RANK_5), Square.valueOf(File.FILE_G, Rank.RANK_5).flipHorizontal());
		Assert.assertEquals(Square.valueOf(File.FILE_H, Rank.RANK_7), Square.valueOf(File.FILE_A, Rank.RANK_7).flipHorizontal());
	}
	
	@Test
	public void testIsLightSquare() {
		Assert.assertTrue(Square.valueOf(File.FILE_A, Rank.RANK_8).isLight());
		Assert.assertFalse(Square.valueOf(File.FILE_B, Rank.RANK_8).isLight());
		Assert.assertTrue(Square.valueOf(File.FILE_C, Rank.RANK_8).isLight());
		Assert.assertFalse(Square.valueOf(File.FILE_A, Rank.RANK_7).isLight());
		Assert.assertTrue(Square.valueOf(File.FILE_B, Rank.RANK_7).isLight());
		Assert.assertFalse(Square.valueOf(File.FILE_C, Rank.RANK_7).isLight());
		Assert.assertTrue(Square.valueOf(File.FILE_A, Rank.RANK_6).isLight());
		Assert.assertFalse(Square.valueOf(File.FILE_B, Rank.RANK_6).isLight());
		Assert.assertTrue(Square.valueOf(File.FILE_C, Rank.RANK_6).isLight());
	}
}
