package chess4j.moves;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import chess4j.board.Board;
import chess4j.board.File;
import chess4j.board.Rank;
import chess4j.board.Square;
import chess4j.io.FenParser;
import chess4j.pieces.Bishop;
import chess4j.pieces.King;
import chess4j.pieces.Knight;
import chess4j.pieces.Pawn;
import chess4j.pieces.Piece;
import chess4j.pieces.Queen;
import chess4j.pieces.Rook;

public class MoveGenTest {

	@Test
	public void testMovesFromInitialPos() {
		Board b = Board.INSTANCE;
		b.resetBoard();
		
		List<Move> moves = MoveGen.genLegalMoves(b);
		Assert.assertEquals(20, moves.size());
		
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_2),Square.valueOf(File.FILE_A, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_2),Square.valueOf(File.FILE_A, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_2),Square.valueOf(File.FILE_B, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_2),Square.valueOf(File.FILE_B, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_C, Rank.RANK_2),Square.valueOf(File.FILE_C, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_C, Rank.RANK_2),Square.valueOf(File.FILE_C, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_2),Square.valueOf(File.FILE_D, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_2),Square.valueOf(File.FILE_D, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_2),Square.valueOf(File.FILE_E, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_2),Square.valueOf(File.FILE_E, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_F, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_F, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_2),Square.valueOf(File.FILE_G, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_2),Square.valueOf(File.FILE_G, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_H, Rank.RANK_2),Square.valueOf(File.FILE_H, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_H, Rank.RANK_2),Square.valueOf(File.FILE_H, Rank.RANK_4))));

		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_1),Square.valueOf(File.FILE_A, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_1),Square.valueOf(File.FILE_C, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_1),Square.valueOf(File.FILE_F, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_1),Square.valueOf(File.FILE_H, Rank.RANK_3))));
	}
	
	@Test
	public void testQueenMoves() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "8/8/3bk3/8/8/2K3Q1/8/8 w - - 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,Queen.class);
		Assert.assertEquals(18, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_G, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_G, Rank.RANK_5))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_G, Rank.RANK_6))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_G, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_G, Rank.RANK_8))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_H, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_H, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_H, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_G, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_G, Rank.RANK_1))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_F, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_E, Rank.RANK_1))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_F, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_E, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_D, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_F, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_E, Rank.RANK_5))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_3),Square.valueOf(File.FILE_D, Rank.RANK_6),Bishop.BLACK_BISHOP)));
	}
	
	@Test
	public void testRookMoves() throws Exception {
		Board b = Board.INSTANCE;
		b.resetBoard();
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,Rook.class);
		Assert.assertEquals(0, moves.size());
		
		FenParser.setPos(b, "8/8/3k1p2/8/3K4/8/1R3r2/8 b - - 0 1");
		moves = filterForPiece(MoveGen.genLegalMoves(b),b,Rook.class);
	
		Assert.assertEquals(10, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_F, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_F, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_F, Rank.RANK_5))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_F, Rank.RANK_1))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_G, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_H, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_E, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_D, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_C, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_F, Rank.RANK_2),Square.valueOf(File.FILE_B, Rank.RANK_2),Rook.WHITE_ROOK)));
	}
	
	@Test
	public void testBishopMoves() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "4k3/2p3P1/8/4B3/4B3/3p1P2/8/4K3 w - - 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,Bishop.class);
		Assert.assertEquals(18, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_F, Rank.RANK_6))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_F, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_G, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_H, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_D, Rank.RANK_4))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_C, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_B, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_A, Rank.RANK_1))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_D, Rank.RANK_6))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_C, Rank.RANK_7),Pawn.BLACK_PAWN)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_F, Rank.RANK_5))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_G, Rank.RANK_6))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_H, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_D, Rank.RANK_3),Pawn.BLACK_PAWN)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_D, Rank.RANK_5))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_C, Rank.RANK_6))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_B, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_A, Rank.RANK_8))));
	}
	
	@Test
	public void testKingMoves() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "8/8/3k4/2n1P3/8/8/3rP3/R3K2R b KQ - 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,King.class);
		Assert.assertEquals(7, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_6),Square.valueOf(File.FILE_D, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_6),Square.valueOf(File.FILE_E, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_6),Square.valueOf(File.FILE_E, Rank.RANK_6))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_6),Square.valueOf(File.FILE_E, Rank.RANK_5),Pawn.WHITE_PAWN)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_6),Square.valueOf(File.FILE_D, Rank.RANK_5))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_6),Square.valueOf(File.FILE_C, Rank.RANK_6))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_6),Square.valueOf(File.FILE_C, Rank.RANK_7))));
		
	}
	
	private List<Move> filterForPiece(List<Move> moves,Board b,Class<?> pieceClass) {
		List<Move> filteredMoves = new ArrayList<Move>();
		for (Move m : moves) {
			Piece p = b.getPiece(m.from());
			if (p.getClass()==pieceClass) {
				filteredMoves.add(m);
			}
		}
		return filteredMoves;
	}
	
	@Test
	public void testKingMoves2() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "8/8/3k4/2n1P3/8/8/3rP3/RN2K2R w KQ - 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,King.class);

		Assert.assertEquals(4, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_1),Square.valueOf(File.FILE_F, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_1),Square.valueOf(File.FILE_F, Rank.RANK_1))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_1),Square.valueOf(File.FILE_D, Rank.RANK_2),Rook.BLACK_ROOK)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_1),Square.valueOf(File.FILE_G, Rank.RANK_1))));
	}
	
	@Test
	public void testKingMoves3() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "k7/8/8/8/8/8/8/7K w - - 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,King.class);

		Assert.assertEquals(3, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_H, Rank.RANK_1),Square.valueOf(File.FILE_G, Rank.RANK_1))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_H, Rank.RANK_1),Square.valueOf(File.FILE_G, Rank.RANK_2))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_H, Rank.RANK_1),Square.valueOf(File.FILE_H, Rank.RANK_2))));
	}

	@Test
	public void testKingMoves4() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "k7/8/8/8/8/8/8/7K b - - 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,King.class);

		Assert.assertEquals(3, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_8),Square.valueOf(File.FILE_A, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_8),Square.valueOf(File.FILE_B, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_8),Square.valueOf(File.FILE_B, Rank.RANK_8))));
	}
	
	@Test
	public void testNoCastleToEscapeCheck() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "8/8/3k4/2N1P3/7q/8/4P3/R3K2R w KQ - 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,King.class);

		Assert.assertEquals(3, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_1),Square.valueOf(File.FILE_F, Rank.RANK_1))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_1),Square.valueOf(File.FILE_D, Rank.RANK_1))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_1),Square.valueOf(File.FILE_D, Rank.RANK_2))));
	}
	
	@Test
	public void testCannotCastleThroughCheck() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "r3k2r/8/8/8/8/5Q2/8/4K3 b kq - 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,King.class);

		Assert.assertEquals(4, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_8),Square.valueOf(File.FILE_D, Rank.RANK_8))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_8),Square.valueOf(File.FILE_D, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_8),Square.valueOf(File.FILE_E, Rank.RANK_7))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_8),Square.valueOf(File.FILE_C, Rank.RANK_8))));
		
	}

	@Test
	public void testPawnMoves() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "2b1k3/PP6/8/3pP3/4P3/8/6P1/4K3 w - d6 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,Pawn.class);
		
		Assert.assertEquals(17, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_7),Square.valueOf(File.FILE_A, Rank.RANK_8),null,Queen.WHITE_QUEEN)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_7),Square.valueOf(File.FILE_A, Rank.RANK_8),null,Rook.WHITE_ROOK)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_7),Square.valueOf(File.FILE_A, Rank.RANK_8),null,Bishop.WHITE_BISHOP)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_A, Rank.RANK_7),Square.valueOf(File.FILE_A, Rank.RANK_8),null,Knight.WHITE_KNIGHT)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_B, Rank.RANK_8),null,Queen.WHITE_QUEEN)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_B, Rank.RANK_8),null,Rook.WHITE_ROOK)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_B, Rank.RANK_8),null,Bishop.WHITE_BISHOP)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_B, Rank.RANK_8),null,Knight.WHITE_KNIGHT)));
		// Capture + Promotion
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_C, Rank.RANK_8),Bishop.BLACK_BISHOP,Queen.WHITE_QUEEN)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_C, Rank.RANK_8),Bishop.BLACK_BISHOP,Rook.WHITE_ROOK)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_C, Rank.RANK_8),Bishop.BLACK_BISHOP,Bishop.WHITE_BISHOP)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_C, Rank.RANK_8),Bishop.BLACK_BISHOP,Knight.WHITE_KNIGHT)));

		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_E, Rank.RANK_6))));
		// EP
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_5),Square.valueOf(File.FILE_D, Rank.RANK_6),Pawn.BLACK_PAWN)));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_E, Rank.RANK_4),Square.valueOf(File.FILE_D, Rank.RANK_5),Pawn.BLACK_PAWN)));

		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_2),Square.valueOf(File.FILE_G, Rank.RANK_3))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_G, Rank.RANK_2),Square.valueOf(File.FILE_G, Rank.RANK_4))));
	}
	
	@Test
	public void testPawnMoves2() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq e3 0 1");
		List<Move> moves = filterForPiece(MoveGen.genLegalMoves(b),b,Pawn.class);
		
		Assert.assertEquals(16, moves.size());
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_7),Square.valueOf(File.FILE_D, Rank.RANK_6))));
		Assert.assertTrue(moves.contains(new Move(Square.valueOf(File.FILE_D, Rank.RANK_7),Square.valueOf(File.FILE_D, Rank.RANK_5))));
	}
}
