package chess4j.board;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class Square {

	private static final Set<Square> squares = new HashSet<Square>();
	private static Map<File,Set<Square>> fileSquares = new HashMap<File,Set<Square>>();
	private static Map<Rank,Set<Square>> rankSquares = new HashMap<Rank,Set<Square>>();

	private final Rank rank;
	private final File file;
	
	private Square(File file,Rank rank) {
		this.file=file;
		this.rank=rank;
	}

	static {
		for (File file : File.values()) {
			Set<Square> squares = new HashSet<Square>();
			for (Rank rank : Rank.values()) {
				squares.add(new Square(file,rank));
			}
			fileSquares.put(file, squares);
		}
		
		for (Rank rank : Rank.values()) {
			Set<Square> squares = new HashSet<Square>();
			for (File file : File.values()) {
				squares.add(new Square(file,rank));
			}
			rankSquares.put(rank, squares);
		}
	}

	static {
		for (Rank rank : Rank.values()) {
			for (File file : File.values()) {
				squares.add(Square.valueOf(file, rank));
			}
		}
	}

	public Rank rank() { return rank; }
	public File file() { return file; }

	@Override
	public String toString() {
		return file.getLabel() + rank.getLabel();
	}

	public boolean isLight() {
		int r = rank.getValue();
		int f = file.getValue();
		return (r%2) == (f%2);
	}
	
	public int value() {
		return rank.getValue()*8 + file.getValue();
	}

	public static Square valueOf(File file,Rank rank) {
		if (file==null || rank==null) return null;
		Set<Square> intersection = new HashSet<Square>(fileSquares.get(file));
		intersection.retainAll(rankSquares.get(rank));
		assert(intersection.size()==1);
		return intersection.iterator().next();
	}

	public static Set<Square> allSquares() {
		return Collections.unmodifiableSet(squares);
	}
	
	public Square flipVertical() {
		return Square.valueOf(file, rank.flip());
	}
	
	public Square flipHorizontal() {
		return Square.valueOf(file.flip(), rank);
	}
	
	public static Set<Square> fileSquares(File file) {
		return Collections.unmodifiableSet(fileSquares.get(file));
	}

	public static Set<Square> rankSquares(Rank rank) {
		return Collections.unmodifiableSet(rankSquares.get(rank));
	}
	
	@Override
	public boolean equals(Object o) {
		if (!(o instanceof Square))
			return false;
		Square sq = (Square)o;
		return sq.file().equals(file) && sq.rank().equals(rank);
	}
	
	@Override
	public int hashCode() {
		int hash = 7;
		hash = 31 * rank.hashCode();
		hash = 31 * file.hashCode();
		return hash;
	}
}
