package chess4j.eval;


import chess4j.Color;
import chess4j.board.Board;
import chess4j.board.Square;
import chess4j.pieces.Bishop;
import chess4j.pieces.Knight;
import chess4j.pieces.Pawn;
import chess4j.pieces.Piece;
import chess4j.pieces.Queen;
import chess4j.pieces.Rook;



public class Eval {

	private static final int QUEEN_VAL  = 900;
	private static final int ROOK_VAL   = 500;
	private static final int KNIGHT_VAL = 300;
	private static final int BISHOP_VAL = 320;
	private static final int PAWN_VAL   = 100;
	

	// A8 ... H8
	//    ...
	// A1 ... H1
	
	private static final int[] knightPST = {
		-5, -5, -5, -5, -5, -5, -5, -5,
		-5,  0, 10, 10, 10, 10,  0, -5,
		-5,  0, 15, 20, 20, 15,  0, -5,
		-5,  5, 10, 15, 15, 10,  5, -5,
		-5,  5, 10, 15, 15, 10,  5, -5,
		-5,  0,  8,  0,  0,  8,  0, -5,
		-5,  0,  0,  5,  5,  0,  0, -5,
		-10,-10, -5, -5, -5, -5,-10,-10 };
	
	private static final int[] pawnPST = {
		 0,  0,  0,  0,  0,  0,  0,  0,
		30, 30, 30, 30, 30, 30, 30, 30,
		14, 14, 14, 18, 18, 14, 14, 14,
		 7,  7,  7, 10, 10,  7,  7,  7,
		 5,  5,  5,  7,  7,  5,  5,  5,
		 3,  3,  3,  5,  5,  3,  3,  3,
		 0,  0,  0, -3, -3,  0,  0,  0,
		 0,  0,  0,  0,  0,  0,  0,  0 }; 
	
	private Eval() { }
	
	public static int eval(Board board) {
		
		int score=0;
		for (Square sq : Square.allSquares()) {
			Piece p = board.getPiece(sq);
			if (p==null) { continue; }
			boolean isWhite = p.isWhite();
			if (p instanceof Pawn) {
				if (p.equals(Pawn.WHITE_PAWN)) {
					score += PAWN_VAL;
					score += pawnPST[sq.value()];
				} else {
					score -= PAWN_VAL;
					score -= pawnPST[sq.flipVertical().value()];
				}
			} else if (p instanceof Rook) {
				score += isWhite ? ROOK_VAL : - ROOK_VAL;
			} else if (p instanceof Knight) {
				if (p.equals(Knight.WHITE_KNIGHT)) {
					score += KNIGHT_VAL;
					score += knightPST[sq.value()];
				} else {
					score -= KNIGHT_VAL;
					score -= knightPST[sq.flipVertical().value()];
				}
			} else if (p instanceof Bishop) {
				score += isWhite ? BISHOP_VAL : -BISHOP_VAL;
			} else if (p instanceof Queen) {
				score += isWhite ? QUEEN_VAL : -QUEEN_VAL;
			}
		}

		return board.getPlayerToMove().equals(Color.WHITE)?score:-score;
	}

}
