package chess4j.utils;

import java.util.List;
import java.util.Set;

import chess4j.board.Board;
import chess4j.board.Square;
import chess4j.moves.Move;
import chess4j.moves.MoveGen;
import chess4j.pieces.Bishop;
import chess4j.pieces.Knight;
import chess4j.pieces.Pawn;
import chess4j.pieces.Piece;
import chess4j.pieces.Queen;
import chess4j.pieces.Rook;

public class GameStatusChecker {
	
	public static GameStatus getGameStatus(Board b) {
		List<Move> moves = MoveGen.genLegalMoves(b);
		if (moves.size()==0) {
			if (b.isPlayerInCheck()) {
				return GameStatus.CHECKMATED;
			} else {
				return GameStatus.STALEMATED;
			}
		}
		
		if (isLackOfMatingMaterial(b)) {
			return GameStatus.DRAW_MATERIAL;
		}

		if (isDrawByRep(b)) {
			return GameStatus.DRAW_REP;
		}
		
		if (b.getFiftyCounter() >= 50) {
			return GameStatus.DRAW_BY_50;
		}
		
		return GameStatus.INPROGRESS;
	}
	
	private static boolean isLackOfMatingMaterial(Board b) {
		
		int numKnights=0,numWhiteSqBishops=0,numBlackSqBishops=0;
		Set<Square> squares = Square.allSquares();
		for (Square sq : squares) {
			Piece p = b.getPiece(sq);
			if (p==null) continue;
			if (p instanceof Pawn) return false;
			if (p instanceof Rook) return false;
			if (p instanceof Queen) return false;
			if (p instanceof Knight) {
				numKnights++;
				if (numKnights > 1) return false;
			}
			if (p instanceof Bishop) {
				if (sq.isLight()) {
					numWhiteSqBishops++;
				} else {
					numBlackSqBishops++;
				}
				
				if (numWhiteSqBishops + numBlackSqBishops > 2) return false;
				if (numWhiteSqBishops==1 && numBlackSqBishops==1) return false;
			}
		}
		
		return true;
	}
	
	private static boolean isDrawByRep(Board b) {
		int reps=0;
		
		Board myBoard = b.deepCopy();
		int numUndos = myBoard.getSizeOfUndoStack();
		for (int i=0;i<numUndos;i++) {
			myBoard.undoLastMove();
			if (myBoard.equalExceptMoveHistory(b, false)) {
				reps++;
			}
		}
		
		return reps >= 2;
	}
}
