package chess4j.board;

import junit.framework.Assert;

import org.junit.Test;

import chess4j.Color;
import chess4j.io.FenParser;
import chess4j.moves.Move;
import chess4j.pieces.Bishop;
import chess4j.pieces.King;
import chess4j.pieces.Knight;
import chess4j.pieces.Pawn;
import chess4j.pieces.Queen;
import chess4j.pieces.Rook;

public class BoardTest {

	@Test
	public void testReset() {
		Board b = Board.INSTANCE;
		b.resetBoard();
		testResetBoard(b);
	}

	private void testResetBoard(Board b) {
		Assert.assertEquals(Rook.WHITE_ROOK,b.getPiece(Square.valueOf(File.FILE_A, Rank.RANK_1)));
		Assert.assertEquals(Knight.WHITE_KNIGHT,b.getPiece(Square.valueOf(File.FILE_B, Rank.RANK_1)));
		Assert.assertEquals(Bishop.WHITE_BISHOP,b.getPiece(Square.valueOf(File.FILE_C, Rank.RANK_1)));
		Assert.assertEquals(Queen.WHITE_QUEEN,b.getPiece(Square.valueOf(File.FILE_D, Rank.RANK_1)));
		Assert.assertEquals(King.WHITE_KING,b.getPiece(Square.valueOf(File.FILE_E, Rank.RANK_1)));
		Assert.assertEquals(Bishop.WHITE_BISHOP,b.getPiece(Square.valueOf(File.FILE_F, Rank.RANK_1)));
		Assert.assertEquals(Knight.WHITE_KNIGHT,b.getPiece(Square.valueOf(File.FILE_G, Rank.RANK_1)));
		Assert.assertEquals(Rook.WHITE_ROOK,b.getPiece(Square.valueOf(File.FILE_H, Rank.RANK_1)));
		for (Square sq : Square.rankSquares(Rank.RANK_2)) {
			Assert.assertEquals(Pawn.WHITE_PAWN,b.getPiece(sq));
		}
		for (Square sq : Square.rankSquares(Rank.RANK_3)) {
			Assert.assertNull(b.getPiece(sq));
		}
		for (Square sq : Square.rankSquares(Rank.RANK_4)) {
			Assert.assertNull(b.getPiece(sq));
		}
		for (Square sq : Square.rankSquares(Rank.RANK_5)) {
			Assert.assertNull(b.getPiece(sq));
		}
		for (Square sq : Square.rankSquares(Rank.RANK_6)) {
			Assert.assertNull(b.getPiece(sq));
		}
		for (Square sq : Square.rankSquares(Rank.RANK_7)) {
			Assert.assertEquals(Pawn.BLACK_PAWN,b.getPiece(sq));
		}
		Assert.assertEquals(Rook.BLACK_ROOK,b.getPiece(Square.valueOf(File.FILE_A, Rank.RANK_8)));
		Assert.assertEquals(Knight.BLACK_KNIGHT,b.getPiece(Square.valueOf(File.FILE_B, Rank.RANK_8)));
		Assert.assertEquals(Bishop.BLACK_BISHOP,b.getPiece(Square.valueOf(File.FILE_C, Rank.RANK_8)));
		Assert.assertEquals(Queen.BLACK_QUEEN,b.getPiece(Square.valueOf(File.FILE_D, Rank.RANK_8)));
		Assert.assertEquals(King.BLACK_KING,b.getPiece(Square.valueOf(File.FILE_E, Rank.RANK_8)));
		Assert.assertEquals(Bishop.BLACK_BISHOP,b.getPiece(Square.valueOf(File.FILE_F, Rank.RANK_8)));
		Assert.assertEquals(Knight.BLACK_KNIGHT,b.getPiece(Square.valueOf(File.FILE_G, Rank.RANK_8)));
		Assert.assertEquals(Rook.BLACK_ROOK,b.getPiece(Square.valueOf(File.FILE_H, Rank.RANK_8)));
		
		Assert.assertTrue(b.hasCastlingRight(CastlingRights.WHITE_KINGSIDE));
		Assert.assertTrue(b.hasCastlingRight(CastlingRights.WHITE_QUEENSIDE));
		Assert.assertTrue(b.hasCastlingRight(CastlingRights.BLACK_KINGSIDE));
		Assert.assertTrue(b.hasCastlingRight(CastlingRights.BLACK_QUEENSIDE));
		
		Assert.assertEquals(Color.WHITE, b.getPlayerToMove());
		Assert.assertNull(b.getEPSquare());
		Assert.assertEquals(0, b.getFiftyCounter());
		Assert.assertEquals(0, b.getMoveCounter());
		
		Assert.assertEquals(Square.valueOf(File.FILE_E, Rank.RANK_1), b.getKingSquare(Color.WHITE));
		Assert.assertEquals(Square.valueOf(File.FILE_E, Rank.RANK_8), b.getKingSquare(Color.BLACK));
	}
	
	@Test
	public void testKingSquares() throws Exception {
		Board b = Board.INSTANCE;
		b.resetBoard();
		Assert.assertEquals(Square.valueOf(File.FILE_E, Rank.RANK_1), b.getKingSquare(Color.WHITE));
		Assert.assertEquals(Square.valueOf(File.FILE_E, Rank.RANK_8), b.getKingSquare(Color.BLACK));

		FenParser.setPos(b, "rnb1kbnr/pp1ppppp/8/8/2p1P3/5N2/PPPNBPPP/R1BQ1RK1 b kq - 0 5");
		Assert.assertEquals(Square.valueOf(File.FILE_G, Rank.RANK_1), b.getKingSquare(Color.WHITE));
		Assert.assertEquals(Square.valueOf(File.FILE_E, Rank.RANK_8), b.getKingSquare(Color.BLACK));

		b.applyMove(new Move(Square.valueOf(File.FILE_E, Rank.RANK_8),Square.valueOf(File.FILE_D, Rank.RANK_8)));
		Assert.assertEquals(Square.valueOf(File.FILE_G, Rank.RANK_1), b.getKingSquare(Color.WHITE));
		Assert.assertEquals(Square.valueOf(File.FILE_D, Rank.RANK_8), b.getKingSquare(Color.BLACK));
	}
	
	@Test
	public void testApplyMoves1() {
		Board b = Board.INSTANCE;
		b.resetBoard();

		Move m = new Move(Square.valueOf(File.FILE_E, Rank.RANK_2), Square.valueOf(File.FILE_E, Rank.RANK_4));
		b.applyMove(m);
		Assert.assertNull(b.getPiece(Square.valueOf(File.FILE_E, Rank.RANK_2)));
		Assert.assertEquals(Pawn.WHITE_PAWN,b.getPiece(Square.valueOf(File.FILE_E, Rank.RANK_4)));
		Assert.assertEquals(Square.valueOf(File.FILE_E, Rank.RANK_3), b.getEPSquare());
		Assert.assertEquals(Color.BLACK, b.getPlayerToMove());
		Assert.assertEquals(1, b.getMoveCounter());
		Assert.assertEquals(0, b.getFiftyCounter());
		
		Move m2 = new Move(Square.valueOf(File.FILE_B, Rank.RANK_8), Square.valueOf(File.FILE_C, Rank.RANK_6));
		b.applyMove(m2);
		Assert.assertNull(b.getPiece(Square.valueOf(File.FILE_B, Rank.RANK_8)));
		Assert.assertTrue(b.isEmpty(Square.valueOf(File.FILE_B, Rank.RANK_8)));
		Assert.assertEquals(Knight.BLACK_KNIGHT,b.getPiece(Square.valueOf(File.FILE_C, Rank.RANK_6)));
		Assert.assertNull(b.getEPSquare());
		Assert.assertEquals(2, b.getMoveCounter());
		Assert.assertEquals(1, b.getFiftyCounter());

		b.undoLastMove();
		b.undoLastMove();
		testResetBoard(b);
	}
	
	@Test
	public void testApplyMoves2() throws Exception {
		Board b = Board.INSTANCE;
		b.resetBoard();
		Board b2 = b.deepCopy();

		Move m = new Move(Square.valueOf(File.FILE_E, Rank.RANK_2), Square.valueOf(File.FILE_E, Rank.RANK_4));
		b.applyMove(m);
		FenParser.setPos(b2, "rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq e3 0 1");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));
		
		b.applyMove(new Move(Square.valueOf(File.FILE_C, Rank.RANK_7), Square.valueOf(File.FILE_C, Rank.RANK_5)));
		FenParser.setPos(b2, "rnbqkbnr/pp1ppppp/8/2p5/4P3/8/PPPP1PPP/RNBQKBNR w KQkq c6 0 2");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));
		
		b.applyMove(new Move(Square.valueOf(File.FILE_G, Rank.RANK_1), Square.valueOf(File.FILE_F, Rank.RANK_3)));
		FenParser.setPos(b2, "rnbqkbnr/pp1ppppp/8/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_D, Rank.RANK_8), Square.valueOf(File.FILE_A, Rank.RANK_5)));
		FenParser.setPos(b2, "rnb1kbnr/pp1ppppp/8/q1p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));
		
		b.applyMove(new Move(Square.valueOf(File.FILE_F, Rank.RANK_1), Square.valueOf(File.FILE_E, Rank.RANK_2)));
		FenParser.setPos(b2, "rnb1kbnr/pp1ppppp/8/q1p5/4P3/5N2/PPPPBPPP/RNBQK2R b KQkq - 3 3");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));
		
		b.applyMove(new Move(Square.valueOf(File.FILE_A, Rank.RANK_5), Square.valueOf(File.FILE_D, Rank.RANK_2),Bishop.WHITE_BISHOP));
		FenParser.setPos(b2, "rnb1kbnr/pp1ppppp/8/2p5/4P3/5N2/PPPqBPPP/RNBQK2R w KQkq - 0 4");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_B, Rank.RANK_1), Square.valueOf(File.FILE_D, Rank.RANK_2),Queen.BLACK_QUEEN));
		FenParser.setPos(b2, "rnb1kbnr/pp1ppppp/8/2p5/4P3/5N2/PPPNBPPP/R1BQK2R b KQkq - 0 4");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_C, Rank.RANK_5), Square.valueOf(File.FILE_C, Rank.RANK_4)));
		FenParser.setPos(b2, "rnb1kbnr/pp1ppppp/8/8/2p1P3/5N2/PPPNBPPP/R1BQK2R w KQkq - 0 5");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_E, Rank.RANK_1), Square.valueOf(File.FILE_G, Rank.RANK_1)));
		FenParser.setPos(b2, "rnb1kbnr/pp1ppppp/8/8/2p1P3/5N2/PPPNBPPP/R1BQ1RK1 b kq - 0 5");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_E, Rank.RANK_8), Square.valueOf(File.FILE_D, Rank.RANK_8)));
		FenParser.setPos(b2, "rnbk1bnr/pp1ppppp/8/8/2p1P3/5N2/PPPNBPPP/R1BQ1RK1 w - - 1 6");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_B, Rank.RANK_2), Square.valueOf(File.FILE_B, Rank.RANK_4)));
		FenParser.setPos(b2, "rnbk1bnr/pp1ppppp/8/8/1Pp1P3/5N2/P1PNBPPP/R1BQ1RK1 b - b3 0 6");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_C, Rank.RANK_4), Square.valueOf(File.FILE_B, Rank.RANK_3)));
		FenParser.setPos(b2, "rnbk1bnr/pp1ppppp/8/8/4P3/1p3N2/P1PNBPPP/R1BQ1RK1 w - - 0 7");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_F, Rank.RANK_1), Square.valueOf(File.FILE_E, Rank.RANK_1)));
		FenParser.setPos(b2, "rnbk1bnr/pp1ppppp/8/8/4P3/1p3N2/P1PNBPPP/R1BQR1K1 b - - 1 7");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_B, Rank.RANK_3), Square.valueOf(File.FILE_B, Rank.RANK_2)));
		FenParser.setPos(b2, "rnbk1bnr/pp1ppppp/8/8/4P3/5N2/PpPNBPPP/R1BQR1K1 w - - 0 8");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_G, Rank.RANK_1), Square.valueOf(File.FILE_H, Rank.RANK_1)));
		FenParser.setPos(b2, "rnbk1bnr/pp1ppppp/8/8/4P3/5N2/PpPNBPPP/R1BQR2K b - - 1 8");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

		b.applyMove(new Move(Square.valueOf(File.FILE_B, Rank.RANK_2), 
				Square.valueOf(File.FILE_A, Rank.RANK_1),
				Rook.WHITE_ROOK,
				Knight.BLACK_KNIGHT));
		FenParser.setPos(b2, "rnbk1bnr/pp1ppppp/8/8/4P3/5N2/P1PNBPPP/n1BQR2K w - - 0 9");
		Assert.assertTrue(b.equalExceptMoveHistory(b2,true));
		Assert.assertTrue(b2.equalExceptMoveHistory(b,true));

	}
	
	@Test
	public void testSwapPlayer() {
		Board b = Board.INSTANCE;
		b.resetBoard();
		b.swapPlayer();
		Assert.assertEquals(Color.BLACK, b.getPlayerToMove());
		b.swapPlayer();
		Assert.assertEquals(Color.WHITE, b.getPlayerToMove());
	}
	
	@Test
	public void testDeepCopy() {
		Board b = Board.INSTANCE;
		Board b2 = b.deepCopy();
		
		Assert.assertEquals(b, b2);
		
		b.swapPlayer();
		Assert.assertFalse(b.equals(b2));
		b.swapPlayer();
		Assert.assertEquals(b, b2);
	}

	@Test
	public void testDeepCopy2() {
		Board b = Board.INSTANCE;
		b.resetBoard();
		b.applyMove(new Move(Square.valueOf(File.FILE_E, Rank.RANK_2),Square.valueOf(File.FILE_E, Rank.RANK_4)));
		Board b2 = b.deepCopy();
		Assert.assertEquals(b, b2);
		
		b.undoLastMove();
		b2.undoLastMove();
		Assert.assertTrue(b.equalExceptMoveHistory(b2, false));
		Assert.assertEquals(b, b2);
	}
	
	@Test
	public void testDeepCopy3() {
		Board b = Board.INSTANCE;
		b.resetBoard();
		Board b2 = b.deepCopy();
		b.applyMove(new Move(Square.valueOf(File.FILE_E, Rank.RANK_2),Square.valueOf(File.FILE_E, Rank.RANK_4)));
		b.undoLastMove();
		Assert.assertEquals(b, b2);
	}

	@Test
	public void testFlipVertical() throws Exception {
		Board b = Board.INSTANCE;
		b.resetBoard();
		Board b2 = b.deepCopy();
		
		b.flipVertical();
		b.flipVertical();
		Assert.assertTrue(b.equals(b2));
		
		FenParser.setPos(b, "7r/R6p/2K4P/5k1P/2p4n/5p2/8/8 w - - 0 1");
		FenParser.setPos(b2, "8/8/5P2/2P4N/5K1p/2k4p/r6P/7R b - - 0 1");
		b.flipVertical();
		// move counter doesn't need to be part of this test
		b.setMoveCounter(b2.getMoveCounter());
		Assert.assertTrue(b.equals(b2));

		// test EP
		FenParser.setPos(b, "rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq e3 0 1");
		FenParser.setPos(b2, "rnbqkbnr/pppp1ppp/8/4p3/8/8/PPPPPPPP/RNBQKBNR w KQkq e6 0 1");
		
		b.flipVertical();
		b.setMoveCounter(b2.getMoveCounter());
		Assert.assertTrue(b.equals(b2));
		
		// test castling
		FenParser.setPos(b, "4k2r/8/8/8/8/8/8/R3K3 b Qk - 0 1");
		FenParser.setPos(b2, "r3k3/8/8/8/8/8/8/4K2R w qK - 0 1");
		b.flipVertical();
		b.setMoveCounter(b2.getMoveCounter());
		Assert.assertTrue(b.equals(b2));
		
	}
}
