package chess4j.search;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.Ignore;
import org.junit.Test;

import chess4j.Constants;
import chess4j.board.Board;
import chess4j.board.File;
import chess4j.board.Rank;
import chess4j.board.Square;
import chess4j.io.FenParser;
import chess4j.moves.Move;
import chess4j.pieces.Pawn;

public class SearchTest {

	@Test
	public void testMateIn1() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "4k3/8/3Q4/2B5/8/8/1K6/8 w - -");
		SearchStats searchStats = new SearchStats();
		searchStats.setStartTime(System.currentTimeMillis());
		searchStats.setStopTime(searchStats.getStartTime()+10000);
		int score = Search.search(new ArrayList<Move>(), -Constants.INFINITY, Constants.INFINITY, b, 2, searchStats);
		Assert.assertEquals(Constants.CHECKMATE-1, score);
	}
	
	@Test
	public void testMateIn1b() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "4K3/8/8/3n2q1/8/8/3k4/8 b - -");
		SearchStats searchStats = new SearchStats();
		searchStats.setStartTime(System.currentTimeMillis());
		searchStats.setStopTime(searchStats.getStartTime()+10000);
		int score = Search.search(new ArrayList<Move>(), -Constants.INFINITY, Constants.INFINITY, b, 2,searchStats);
		Assert.assertEquals(Constants.CHECKMATE-1, score);
	}

	@Test
	public void testMateIn2() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "r1bq2r1/b4pk1/p1pp1p2/1p2pP2/1P2P1PB/3P4/1PPQ2P1/R3K2R w - -");
		SearchStats searchStats = new SearchStats();
		searchStats.setStartTime(System.currentTimeMillis());
		searchStats.setStopTime(searchStats.getStartTime()+10000);
		int score = Search.search(new ArrayList<Move>(), -Constants.INFINITY, Constants.INFINITY, b, 4, searchStats);
		Assert.assertEquals(Constants.CHECKMATE-3, score);
	}

	@Ignore // takes too long
	@Test
	public void testMateIn3() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "r5rk/5p1p/5R2/4B3/8/8/7P/7K w - -");
		SearchStats searchStats = new SearchStats();
		int score = Search.search(new ArrayList<Move>(), -Constants.INFINITY, Constants.INFINITY, b, 6, searchStats);
		Assert.assertEquals(Constants.CHECKMATE-5, score);
	}
	
	@Test
	public void testStaleMate() throws Exception {
		Board b = Board.INSTANCE;
		FenParser.setPos(b, "8/6p1/5p2/5k1K/7P/8/8/8 w - -");
		SearchStats searchStats = new SearchStats();
		List<Move> pv = new ArrayList<Move>();
		searchStats.setStartTime(System.currentTimeMillis());
		searchStats.setStopTime(searchStats.getStartTime()+10000);
		int score = Search.search(pv, -Constants.INFINITY, Constants.INFINITY, b, 1, searchStats);
		Assert.assertEquals(0, score);
		Assert.assertEquals(0, pv.size());
	}

	@Test
	public void testSearchLastPVFirst() throws Exception {
		Board b = Board.INSTANCE;
		b.resetBoard();
		
		// create an artificial PV and ensure it is searched first.
		Move c2c4 = new Move(Square.valueOf(File.FILE_C, Rank.RANK_2),Square.valueOf(File.FILE_C, Rank.RANK_4));
		Move b7b5 = new Move(Square.valueOf(File.FILE_B, Rank.RANK_7),Square.valueOf(File.FILE_B, Rank.RANK_5));
		Move c4b5 = new Move(Square.valueOf(File.FILE_C, Rank.RANK_4),Square.valueOf(File.FILE_B, Rank.RANK_5),Pawn.BLACK_PAWN);
		Move g8h6 = new Move(Square.valueOf(File.FILE_G, Rank.RANK_8),Square.valueOf(File.FILE_H, Rank.RANK_6));
		List<Move> lastPV = new ArrayList<Move>();
		lastPV.add(c2c4);
		lastPV.add(b7b5);
		lastPV.add(c4b5);
		lastPV.add(g8h6);
		SearchStats searchStats = new SearchStats();
		searchStats.setStartTime(System.currentTimeMillis());
		searchStats.setStopTime(searchStats.getStartTime()+10000);
		searchStats.setLastPV(lastPV);

		List<Move> pv = new ArrayList<Move>();
		Search.search(pv, -Constants.INFINITY, Constants.INFINITY, b, 5, searchStats);
		
		Assert.assertEquals(5, searchStats.getFirstLine().size());
		Assert.assertEquals(lastPV, searchStats.getFirstLine().subList(0, 4));
	}
	
	@Test
	public void testAbortSearch() throws Exception {
		Board b = Board.INSTANCE;
		b.resetBoard();

		SearchStats searchStats = new SearchStats();
		searchStats.setStartTime(System.currentTimeMillis());
		searchStats.setStopTime(searchStats.getStartTime()+10000);
		Search.setAbortSearch(true);
		Search.search(new ArrayList<Move>(), -Constants.INFINITY, Constants.INFINITY, b, 1, searchStats);
		Assert.assertEquals(21, searchStats.getNodes());

		searchStats = new SearchStats();
		searchStats.setStartTime(System.currentTimeMillis());
		searchStats.setStopTime(searchStats.getStartTime()+10000);
		Search.search(new ArrayList<Move>(), -Constants.INFINITY, Constants.INFINITY, b, 3, searchStats);
		Assert.assertEquals(4, searchStats.getNodes()); // 1 + 3 (down left side of the tree)
		
		Search.setAbortSearch(false);
	}
}
